/* eslint-disable no-redeclare */
/*
Trimmed version of "ip" package (https://www.npmjs.com/package/ip) that fixes an error when running in Node.js v18.

This software is licensed under the MIT License.

Copyright Fedor Indutny, 2012.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
import os from 'os';
function _resolveFamily(family) {
    return typeof family === 'number' ? 'ipv' + family : family.toLowerCase();
}
function _normalizeFamily(family) {
    return family ? _resolveFamily(family) : 'ipv4';
}
function isPrivate(addr) {
    return /^(::f{4}:)?10\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})$/i
        .test(addr) ||
        /^(::f{4}:)?192\.168\.([0-9]{1,3})\.([0-9]{1,3})$/i.test(addr) ||
        /^(::f{4}:)?172\.(1[6-9]|2\d|30|31)\.([0-9]{1,3})\.([0-9]{1,3})$/i
            .test(addr) ||
        /^(::f{4}:)?127\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})$/i.test(addr) ||
        /^(::f{4}:)?169\.254\.([0-9]{1,3})\.([0-9]{1,3})$/i.test(addr) ||
        /^f[cd][0-9a-f]{2}:/i.test(addr) ||
        /^fe80:/i.test(addr) ||
        /^::1$/.test(addr) ||
        /^::$/.test(addr);
}
function isPublic(addr) {
    return !isPrivate(addr);
}
function isLoopback(addr) {
    return /^(::f{4}:)?127\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/
        .test(addr) ||
        /^fe80::1$/.test(addr) ||
        /^::1$/.test(addr) ||
        /^::$/.test(addr);
}
function loopback(family) {
    //
    // Default to `ipv4`
    //
    family = _normalizeFamily(family);
    if (family !== 'ipv4' && family !== 'ipv6') {
        throw new Error('family must be ipv4 or ipv6');
    }
    return family === 'ipv4' ? '127.0.0.1' : 'fe80::1';
}
//
// ### function address (name, family)
// #### @name {string|'public'|'private'} **Optional** Name or security
//      of the network interface.
// #### @family {ipv4|ipv6} **Optional** IP family of the address (defaults
//      to ipv4).
//
// Returns the address for the network interface on the current system with
// the specified `name`:
//   * String: First `family` address of the interface.
//             If not found see `undefined`.
//   * 'public': the first public ip address of family.
//   * 'private': the first private ip address of family.
//   * undefined: First address with `ipv4` or loopback address `127.0.0.1`.
//
export function address(name, family) {
    var interfaces = os.networkInterfaces();
    var all;
    //
    // Default to `ipv4`
    //
    family = _normalizeFamily(family);
    //
    // If a specific network interface has been named,
    // return the address.
    //
    if (name && name !== 'private' && name !== 'public') {
        var res = interfaces[name].filter(function (details) {
            var itemFamily = _resolveFamily(details.family);
            return itemFamily === family;
        });
        if (res.length === 0)
            return undefined;
        return res[0].address;
    }
    var all = Object.keys(interfaces).map(function (nic) {
        //
        // Note: name will only be `public` or `private`
        // when this is called.
        //
        var addresses = interfaces[nic].filter(function (details) {
            details.family = _resolveFamily(details.family);
            if (details.family !== family || isLoopback(details.address)) {
                return false;
            }
            else if (!name) {
                return true;
            }
            return name === 'public' ? isPrivate(details.address) :
                isPublic(details.address);
        });
        return addresses.length ? addresses[0].address : undefined;
    }).filter(Boolean);
    return !all.length ? loopback(family) : all[0];
}
